/*
 * MathPlanner 3.1 - Mathematical design tool.
 * Copyright(C) 2002  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */

#ifndef _MATHPLANNER_APP_CONTROL_H
#define _MATHPLANNER_APP_CONTROL_H

#include <qfont.h>
#include <qcolor.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qwidgetstack.h>
#include <qaccel.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>
#include <qtoolbutton.h>
#include <qpixmap.h>
#include <qtimer.h>

class DataStorage;
class string_object;
class m_function_object;
class MathWindow;
class m_base_object;
class Paper;


class ApplicationControl : public QObject
{
	Q_OBJECT

       	public:
       	        ApplicationControl(MathWindow *parent);
   	
       	void    CreateFunctionMenu(QStringList);
        QColor  GetColor(int id);
        QFont   GetFont(int id);
        int     GetFontSize(int id);
				int		  CenterLine(string_object *);    	
       	void    CreateControlMenu(m_base_object *);    	     	 	
       	int     ZoomFactor();
       	void    ZoomFactor(int);
       	void    AddError(QString);    	   	
				void    Read();
        void    MessageToActiveObject(int a,int b,QString c);
        Paper   *GetActivePaper();

				m_function_object *GetActiveFunction();
				

        private:


        void    Command(int);
        void    ControlBarSetup();
       	void    CreateOperatorBar();
        void    CreateComplexBar();
        void    SetupFonts();

				void    RecFunctionStatus();
				void    SendFunctionStatus();


       	private slots:
       	
       	void TimeOut();
       	void NextZoomFactor();
       	void PrevZoomFactor();
       	void Send(int);
       	void Send();
       	void ZoomChanged(int);
       	void SetFunction(int);
        void SetFont(int);
       	
       	// ------------------------------
       	void TPiiConst();
       	void TSqrt();
        void TDive();
        void TPower();
        void TLog();
				void Tln();
				void TExp();
				void Te();
				void TMin();
				void TMax();
	
				void TSin();
        void TCos();
        void TTan();
        void TCot();
        void TaSin();
        void TaCos();
        void TaTan();
        void TaCot();

        void TFact();
        void TFrac();
        void TInt();

        void TAbs();
				void TParA();
				void TParB();
				void TParC();
	
				void TSigma();
				void TPii();
				void TDef();
        void TDef2();
        void TDef3();

        void TImaginary();
				void TVectI();
				void TVectJ();
				void TVectK();

				void TArg();
				void TModul();
				void TRez();
				void TImz();
        void TCAng();

        void TVLen();
        void TVCross();
        void TConj();

        void TUnit();


        public:
	
        bool DocumentChanged; //Check this
        QString SelectedFunction; //Clean up
        QPopupMenu *FunctionMenu; //Clean up

        QTimer Timer;  //Clean up
        QPixmap DragImage; //Clean up

        class DataStorage *Clipboard;
        class function_serv *function_server;
				class int_server *integer_server;
				class config_file_reader *Prefs;
        class setup *Setup;
	      class mpl_unit *Unit;



       	private:

      	class MathWindow *parent;
       	class base_object_message *base_om[50];
       	
        int zoomfactor;


				QPixmap pm1,pm2,pm3;
				QPixmap pm4,pm5,pm6;
				
				QPixmap colorbuttonpm;
				
				QWidgetStack *wstack;
				
				
				QToolButton *opr_b[50];
        QToolButton *opr_c[50];
				QToolButton *tba1,*tba2;
				QPushButton *colorbutton,*pus1,*pus2,*rational;

				QColor color[16];
  			QFont font[16];
  			int fontsize[16];
        int centerline;

				QComboBox *expmodecombo,*fontsizecombo,*digitcount,*zoomcombo,*answercombo;
    		QToolBar *ControlBar,*ObjectBar,*OperatorBar,*ComplexBar;

};

#endif
