/*
 * MathPlanner 3.1 - Mathematical design tool.
 * Copyright(C) 2002  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */

#include "AppControl.h"
#include "Paper.h"
#include "Datastore.h"
#include "Frame.h"


MathFrame::MathFrame(DataStorage *msg,ApplicationControl *a,Paper *p)
{
  	AppControl=a;
	  paper=p;
	
  	ObjectType=msg->ReadInt("Object");
  	msg->ReadQRect("Bounds",&bounds);
  	msg->ReadQColor("MF:C",&color);
    ascent=msg->ReadInt("MF:A");
  	frame_enabled=msg->ReadInt("MF:FE");
  	resize_enabled=msg->ReadInt("MF:RE");

  	old_bounds=bounds;
}

MathFrame::~MathFrame()
{

}


DataStorage *MathFrame::BuildStorage()
{
    DataStorage *msg=new DataStorage();

    msg->AddInt("Object",ObjectType);
  	msg->AddQRect("Bounds",&bounds);
  	msg->AddQColor("MF:C",color);
  	msg->AddInt("MF:FE",(int)frame_enabled);
  	msg->AddInt("MF:RE",(int)resize_enabled);
    msg->AddInt("MF:A",ascent);

  	return(msg);
}


MathFrame::MathFrame(int type,QRect b,ApplicationControl *a,Paper *p,int as)
{
  ObjectType=type;
	old_bounds=bounds=b;
	AppControl=a;
	paper=p;
  ascent=as;
}


QRect MathFrame::Bounds()
{
	return(bounds);
}

QRect MathFrame::OldBounds()
{
	return(old_bounds);
}

void MathFrame::Enable(bool f,bool r,bool a)
{
	frame_enabled=f;
	resize_enabled=r;
	always=a;
}


bool MathFrame::IsEmpty()
{
	return(empty);	
}


void MathFrame::SetColor(QColor c)
{
	color=c;
}


void MathFrame::SetBounds(QRect b,int as)
{
  old_bounds=bounds|old_bounds;
	bounds=b;
  ascent=as;
}


void MathFrame::MoveTo(QPoint p)
{	
 		bounds.moveTopLeft(p-QPoint(0,ascent));
}

QPoint MathFrame::Position()
{
    return(bounds.topLeft()+QPoint(0,ascent));
}


void MathFrame::CalculateBounds()
{
     old_bounds=bounds|old_bounds;
}

void MathFrame::MousePressed(QPoint p)
{

}


bool  MathFrame::IsInBounds(QPoint p)
{
	return(bounds.contains(p));
}


bool  MathFrame::IsInResizeArea(QPoint p)
{
  if (resize_enabled) {
        resizerect.setSize(QSize(10,10));
        resizerect.moveBottomRight(bounds.bottomRight());
	return(resizerect.contains(p));
	}
	return(false);
}


bool  MathFrame::HasFocus()
{
	if (paper->GetFocus()==this) return(true);
	return(false);
}

void  MathFrame::UnFocus()
{
	//paper->FocusNone();	
  paper->UnFocusCurrentFunction();
}


void MathFrame::Draw(QPainter *paint)
{
  old_bounds=bounds;

  if (frame_enabled) {
   	 paint->setPen(color);
     IsInResizeArea(QPoint(0,0));
	   if (HasFocus() || always) paint->drawRect(bounds);
	   if (HasFocus() && resize_enabled) paint->drawRect(resizerect);
	}
}
	
	
	
	
	




