/*
 * MathPlanner 3.0.4 - Mathematical design tool.
 * Copyright(C) 2002  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */


#include "Message.moc"
#include "Datastore.h"
#include "Math.moc"
#include "AppControl.h"
#include "Paper.h"
#include "Error.h"
#include "ListOfGreekAlphabets.h"
#include "Setup.h"
#include "ConfigReader.h"
#include "Extern.h"

#include <qfiledialog.h>
#include <qprinter.h>
#include <qvbox.h>
#include <stdlib.h>



MathWindow::MathWindow()
:QMainWindow(0,"MathPlanner")
{
	int i;
	ApplicationCaption=QString("MathPlanner ")+QString(_MPL_VERSION);
	CurrentDocumentName="";

	for(i=0;i<32;i++) paper[i]=NULL;
  Active_paper=0;

	datastore=0;

  Bug("Launching AppControl..."); 	
  AppControl=new ApplicationControl(this);

  AppControl->AddError("");	

	message=new Message(this);

	mainbox=new QVBox(this);	
	mainbox->resize(700,550);
	mainbox->show();
	
 	scrollview=new QScrollView(mainbox);
 			
 	setCentralWidget(mainbox);
		
  setCaption(ApplicationCaption);

  BuildMenu();
   	 	
}



class Paper *MathWindow::GetActivePaper()
{
     	return(paper[Active_paper]);
}


void MathWindow::Init()
{  	
     NewPage();
}


QString MathWindow::Browser()
{  	
     return(AppControl->Prefs->String("WEBBROWSER"));
}

int MathWindow::Style()
{  	
     return(AppControl->Prefs->Int("GUITheme"));	
}



void MathWindow::BuildMenu()
{
	ToolBarB=new QToolBar(this);




	if (!pm1.load(image_path+QString("/Uusi.xpm"))) ErrorReport("BuildMenu","No Image Uusi");
	if (!pm2.load(image_path+QString("/Lataa.xpm"))) ErrorReport("BuildMenu","No Image Lataa");
	if (!pm3.load(image_path+QString("/Tallenna.xpm"))) ErrorReport("BuildMenu","No Image Tallenna");
	if (!pm4.load(image_path+QString("/Preferences.xpm"))) ErrorReport("BuildMenu","No Image");
	
	if (!pm8.load(image_path+QString("/Zoom_plus.xpm"))) ErrorReport("BuildMenu","No Image");
	if (!pm9.load(image_path+QString("/Zoom_minus.xpm"))) ErrorReport("BuildMenu","No Image");
	
	if (!pm10.load(image_path+QString("/Print.xpm"))) ErrorReport("BuildMenu","No Image");
	//if (!pm13.load(image_path+QString("/Functions.xpm"))) ErrorReport("BuildMenu","No Image");
	if (!pm14.load(image_path+QString("/Calculate.xpm"))) ErrorReport("BuildMenu","No Image");
	
	if (!pm15.load(image_path+QString("/Copy.xpm"))) ErrorReport("BuildMenu","No Image");
	if (!pm16.load(image_path+QString("/Cut.xpm"))) ErrorReport("BuildMenu","No Image");
	if (!pm17.load(image_path+QString("/Paste.xpm"))) ErrorReport("BuildMenu","No Image");
		
	if (!pm18.load(image_path+QString("/SelAll.xpm"))) ErrorReport("BuildMenu","No Image");
	if (!pm19.load(image_path+QString("/SelNone.xpm"))) ErrorReport("BuildMenu","No Image");
	if (!pm20.load(image_path+QString("/Delete.xpm"))) ErrorReport("BuildMenu","No Image");
	
	if (!pm21.load(image_path+QString("/Graph.xpm"))) ErrorReport("BuildMenu","No Image");
	if (!pm22.load(image_path+QString("/Textframe.xpm"))) ErrorReport("BuildMenu","No Image");
	
	
			
	                                                                  	
	                                                                 	
	pop1=new QPopupMenu(this);
	pop1->insertItem(pm1,tr("New..."),message,SLOT(Uusi()),CTRL+Key_N);
	pop1->insertItem(pm2,tr("Open..."),message,SLOT(Avaa()),CTRL+Key_O);	
	pop1->insertSeparator();
	pop1->insertItem(pm3,tr("Save"),this,SLOT(SaveCurrent()),CTRL+Key_S,0);
	pop1->insertItem(pm3,tr("Save As..."),message,SLOT(Tallenna()));	
	pop1->insertSeparator();
	pop1->insertItem(pm10,tr("Print..."),this,SLOT(Print()),CTRL+Key_P);
	pop1->insertSeparator();
  pop1->insertItem(tr("Preferences..."),this,SLOT(Prefs()));
  pop1->insertSeparator();
	pop1->insertItem(tr("Quit"),this,SLOT(close()));	
	
	menuBar()->insertItem(tr("  File  "),pop1);
	



	pop2=new QPopupMenu(this);
	pop2->insertItem(pm16,tr("Cut"),this,SLOT(Cut()),CTRL+Key_X);
	pop2->insertItem(pm15,tr("Copy"),this,SLOT(Copy()),CTRL+Key_C);	
	pop2->insertItem(pm17,tr("Paste"),this,SLOT(Paste()),CTRL+Key_V);
	pop2->insertSeparator();
	pop2->insertItem(pm18,tr("Select All"),this,SLOT(SelectAll()),CTRL+Key_A);	
	pop2->insertItem(pm19,tr("Clear"),this,SLOT(SelectNone()));	
	pop2->insertSeparator();
	pop2->insertItem(pm20,tr("Delete"),this,SLOT(DeleteFunction()),CTRL+Key_D);	
	
	menuBar()->insertItem(tr("  Edit  "),pop2);


	pop3=new QPopupMenu(this);
	pop3->insertItem(pm15,tr("Copy"),this,SLOT(DragObject()));
	pop3->insertItem(pm17,tr("Paste"),this,SLOT(DropObject()));
	
	menuBar()->insertItem(tr("  Global_edit "),pop3);	
	
  QPopupMenu *pop4=new QPopupMenu(this);
  QPopupMenu *pop5=new QPopupMenu(this);

	pop4->insertItem(tr("Upper Case"),pop5);
  pop4->insertSeparator();
  pop4->insertItem(QChar(GREEK_alfa)+QString("  Alfa"),GREEK_alfa);
	pop4->insertItem(QChar(GREEK_beta)+QString("  Beta"),GREEK_beta);
	pop4->insertItem(QChar(GREEK_gamma)+QString("  Gamma"),GREEK_gamma);
	pop4->insertItem(QChar(GREEK_delta)+QString("  Delta"),GREEK_delta);
	pop4->insertItem(QChar(GREEK_epsilon)+QString("  Epsilon"),GREEK_epsilon);
	pop4->insertItem(QChar(GREEK_zeeta)+QString("  Zeeta"),GREEK_zeeta);
	pop4->insertItem(QChar(GREEK_eeta)+QString("  Eeta"),GREEK_eeta);
	pop4->insertItem(QChar(GREEK_theeta)+QString("  Theeta"),GREEK_theeta);
	pop4->insertItem(QChar(GREEK_ioota)+QString("  Ioota"),GREEK_ioota);
	pop4->insertItem(QChar(GREEK_kappa)+QString("  Kappa"),GREEK_kappa);
	pop4->insertItem(QChar(GREEK_lambda)+QString("  Lambda"),GREEK_lambda);
	pop4->insertItem(QChar(GREEK_myy)+QString("  Myy"),GREEK_myy);
	pop4->insertItem(QChar(GREEK_nyy)+QString("  Nyy"),GREEK_nyy);
	pop4->insertItem(QChar(GREEK_ksii)+QString("  Ksii"),GREEK_ksii);
	pop4->insertItem(QChar(GREEK_omikron)+QString("  Omikron"),GREEK_omikron);
	pop4->insertItem(QChar(GREEK_pii)+QString("  Pii"),GREEK_pii);
	pop4->insertItem(QChar(GREEK_rhoo)+QString("  Rhoo"),GREEK_rhoo);
	pop4->insertItem(QChar(GREEK_sigma)+QString("  Sigma"),GREEK_sigma);
	pop4->insertItem(QChar(GREEK_tau)+QString("  Tau"),GREEK_tau);
	pop4->insertItem(QChar(GREEK_ypsilon)+QString("  Ypsilon"),GREEK_ypsilon);
	pop4->insertItem(QChar(GREEK_fii)+QString("  Fii"),GREEK_fii);
	pop4->insertItem(QChar(GREEK_khii)+QString("  Khii"),GREEK_khii);
	pop4->insertItem(QChar(GREEK_psii)+QString("  Psii"),GREEK_psii);
	pop4->insertItem(QChar(GREEK_omega)+QString("  Omega"),GREEK_omega);

  pop5->insertItem(QChar(GREEK_ALFA)+QString("  Alfa"),GREEK_ALFA);
	pop5->insertItem(QChar(GREEK_BETA)+QString("  Beta"),GREEK_BETA);
	pop5->insertItem(QChar(GREEK_GAMMA)+QString("  Gamma"),GREEK_GAMMA);
	pop5->insertItem(QChar(GREEK_DELTA)+QString("  Delta"),GREEK_DELTA);
	pop5->insertItem(QChar(GREEK_EPSILON)+QString("  Epsilon"),GREEK_EPSILON);
	pop5->insertItem(QChar(GREEK_ZEETA)+QString("  Zeeta"),GREEK_ZEETA);
	pop5->insertItem(QChar(GREEK_EETA)+QString("  Eeta"),GREEK_EETA);
	pop5->insertItem(QChar(GREEK_THEETA)+QString("  Theeta"),GREEK_THEETA);
	pop5->insertItem(QChar(GREEK_IOOTA)+QString("  Ioota"),GREEK_IOOTA);
	pop5->insertItem(QChar(GREEK_KAPPA)+QString("  Kappa"),GREEK_KAPPA);
	pop5->insertItem(QChar(GREEK_LAMBDA)+QString("  Lambda"),GREEK_LAMBDA);
	pop5->insertItem(QChar(GREEK_MYY)+QString("  Myy"),GREEK_MYY);
	pop5->insertItem(QChar(GREEK_NYY)+QString("  Nyy"),GREEK_NYY);
	pop5->insertItem(QChar(GREEK_KSII)+QString("  Ksii"),GREEK_KSII);
	pop5->insertItem(QChar(GREEK_OMIKRON)+QString("  Omikron"),GREEK_OMIKRON);
	pop5->insertItem(QChar(GREEK_PII)+QString("  Pii"),GREEK_PII);
	pop5->insertItem(QChar(GREEK_RHOO)+QString("  Rhoo"),GREEK_RHOO);
	pop5->insertItem(QChar(GREEK_SIGMA)+QString("  Sigma"),GREEK_SIGMA);
	pop5->insertItem(QChar(GREEK_TAU)+QString("  Tau"),GREEK_TAU);
	pop5->insertItem(QChar(GREEK_YPSILON)+QString("  Ypsilon"),GREEK_YPSILON);
	pop5->insertItem(QChar(GREEK_FII)+QString("  Fii"),GREEK_FII);
	pop5->insertItem(QChar(GREEK_KHII)+QString("  Khii"),GREEK_KHII);
	pop5->insertItem(QChar(GREEK_PSII)+QString("  Psii"),GREEK_PSII);
	pop5->insertItem(QChar(GREEK_OMEGA)+QString("  Omega"),GREEK_OMEGA);

  menuBar()->insertItem(tr("  Symbols  "),pop4);	

  connect(pop4,SIGNAL(activated(int)),SLOT(SymbolSelected(int)));
  connect(pop5,SIGNAL(activated(int)),SLOT(SymbolSelected(int)));

  menuBar()->insertItem(tr("  Functions  "),AppControl->FunctionMenu);

  QPopupMenu *pop6=new QPopupMenu(this);
  pop6->insertItem(tr("User manual"),this,SLOT(Help()));
  pop6->insertSeparator();
  pop6->insertItem(tr("About..."),this,SLOT(About()));
  menuBar()->insertItem(tr("  Help  "),pop6);	

	
	tb1=new QToolButton(pm1,QString(tr("New")),QString::null,message,SLOT(Uusi()),ToolBarB);
	tb2=new QToolButton(pm2,QString(tr("Open")),QString::null,message,SLOT(Avaa()),ToolBarB);
	tb3=new QToolButton(pm3,QString(tr("Save")),QString::null,message,SLOT(Tallenna()),ToolBarB);
	ToolBarB->addSeparator();
	tb4=new QToolButton(pm4,QString(tr("Preferences")),QString::null,this,SLOT(Prefs()),ToolBarB);   // Removed MPL 3.0-Pre1
	tb5=new QToolButton(pm10,QString(tr("Print")),QString::null,this,SLOT(Print()),ToolBarB);
	ToolBarB->addSeparator();
	tb6=new QToolButton(pm15,QString(tr("Copy")),QString::null,this,SLOT(Copy()),ToolBarB);
	tb7=new QToolButton(pm16,QString(tr("Cut")),QString::null,this,SLOT(Cut()),ToolBarB);
	tb8=new QToolButton(pm17,QString(tr("Paste")),QString::null,this,SLOT(Paste()),ToolBarB);
	ToolBarB->addSeparator();
	tb9=new QToolButton(pm18,QString(tr("Select All")),QString::null,this,SLOT(SelectAll()),ToolBarB);
	tb10=new QToolButton(pm19,QString(tr("Select none")),QString::null,this,SLOT(SelectNone()),ToolBarB);
	tb11=new QToolButton(pm20,QString(tr("Delete function")),QString::null,this,SLOT(DeleteFunction()),ToolBarB);
	ToolBarB->addSeparator();
	tb12=new QToolButton(pm8,QString(tr("Zoom In")),QString::null,AppControl,SLOT(NextZoomFactor()),ToolBarB);	
	tb13=new QToolButton(pm9,QString(tr("Zoom Out")),QString::null,AppControl,SLOT(PrevZoomFactor()),ToolBarB);
	ToolBarB->addSeparator();
	tb18=new QToolButton(pm21,QString(tr("Graph setup (not operational)")),QString::null,this,SLOT(AddGraph()),ToolBarB);	
	tb19=new QToolButton(pm22,QString(tr("Text frame (not operational)")),QString::null,this,SLOT(None()),ToolBarB);
	ToolBarB->addSeparator();
	tb17=new QToolButton(pm14,QString(tr("Calculate / Update")),QString::null,this,SLOT(Calculate()),ToolBarB);



	addToolBar(ToolBarB,0,Top,true);
	
	setDockEnabled(ToolBarB,Top,true);
	
	setToolBarsMovable(true);	
	setDockEnabled(Left,true);
			
}

		

void MathWindow::resizeEvent( QResizeEvent *R )
{
	
}




Message::Message(MathWindow *p)
:QObject(p)
{
parent=p;
}




void Message::Uusi()
{
        parent->CurrentDocumentName="";
        Paper *pap=parent->GetActivePaper();
        if (pap) pap->New();
}



void Message::Avaa()
{
	QFileDialog *dia=new QFileDialog(home_path);
   	QString s( dia->getOpenFileName(home_path, "MathPlanner (*.mpl2)", parent ) );
	if ( s.isEmpty() ) return;
	parent->Open(s);
	delete dia;
}



void Message::Tallenna()
{
	QFileDialog *dia=new QFileDialog(home_path);
	QString fileName = dia->getSaveFileName(home_path+QString("/.mpl2"), "MathPlanner (*.mpl2)", parent );
	if ( !fileName.isNull() ) {
 	parent->Save(fileName);
	}
	delete dia;
}



void MathWindow::SaveCurrent()
{
 	if (!CurrentDocumentName.isEmpty()) Save(CurrentDocumentName);
}



void MathWindow::Rescue()
{
	Save(config_path+QString("/MathPlanner-autosave.mpl2"));
}



void MathWindow::None()
{

}


void MathWindow::Help()
{
QString exe(webbrowser+" "+doc_dir+"/Doc.htm");
system((const char *)exe.latin1());
}



void MathWindow::Prefs()
{
AppControl->Setup->Show();
}



void MathWindow::Print()
{
QPrinter *printer=new QPrinter();
Paper *pap=GetActivePaper();
if (pap) {

  pap->FocusNone();

	if ( printer->setup( this ) ) {
	        QPainter paint;
	        paint.begin(printer);
	        paint.setWindow(paper[0]->Scale(100,AppControl->Prefs->Rect("PaperWidth","PaperHeight")));
		pap->Draw(&paint,QRect(0,0,0,0),true);
		paint.end();	
	}
}

delete printer;

}



void MathWindow::SymbolSelected(int id)
{
		Paper *pap=GetActivePaper();
    QKeyEvent key(QEvent::KeyPress,0,id,0,QString(QChar(id)));
    if (pap) pap->keyPressEvent(&key);
}




void MathWindow::AddGraph()
{
  //Paper *pap=GetActivePaper();
  //if (pap) pap->AddGraph();
}




void MathWindow::DragObject()
{
 Paper *pap=GetActivePaper();
 if (pap) pap->DragObject();
}





void MathWindow::DropObject()
{
  Paper *pap=GetActivePaper();
  if (pap) pap->DropObject();
}



void MathWindow::DeleteFunction()
{
   Paper *pap=GetActivePaper();
   if (pap) pap->DeleteCurrentFunction();
}



void MathWindow::SelectNone()
{
   Paper *pap=GetActivePaper();
   if (pap) pap->FocusNone();
}



void MathWindow::SelectAll()
{
   Paper *pap=GetActivePaper();
   if (pap) pap->CommandToString(STRING_COMMAND_SELECT_ALL,0);
}



void MathWindow::Copy()
{
   Paper *pap=GetActivePaper();
   if (pap) pap->CommandToString(STRING_COMMAND_COPY,0);
}



void MathWindow::Paste()
{
   Paper *pap=GetActivePaper();
   if (pap) pap->CommandToString(STRING_COMMAND_PASTE,0);
}

void MathWindow::Cut()
{
   Paper *pap=GetActivePaper();
   if (pap) pap->CommandToString(STRING_COMMAND_CUT,0);
}



int MathWindow::GetFreePage()
{
  	int i;
	for(i=0;i<32;i++) {
	    if (paper[i]==NULL) return(i);
	}
	return(-1);
}



QPoint MathWindow::PaperPosition(int i)        // What is this ???????
{
     int w=AppControl->Prefs->Int("PaperWidth")+10;
     int h=AppControl->Prefs->Int("PaperHeight")+10;

     int hp=i/4;
     int wp=i-(hp*4);

     return(QPoint(10+wp*w,10*hp*h));
}


void MathWindow::About()
{
QMessageBox::about( this, QString("About MathPlanner ")+QString(_MPL_VERSION)+QString("                     ")+QString(_MPL_DATE),
           "MathPlanner Is created by Jarmo Nikkanen. MathPlanner can be freely\n"
           "distributed under GPL. You should have received a copy of the GNU \n"
           "General Public License with this program.\n\n"
           "Comments, Ideas, Bug reports are received at MathPlanner home page\n\n"
           "http://koti.mbnet.fi/jarmonik\n\n"
           "or via e-mail: jarmonik@mbnet.fi\n\n"
           "I hope you enjoy using this program\n");
}

	

void MathWindow::NewPage()
{
   Bug("Adding a Page");
   int i=GetFreePage();
   if (i>=0) {
     Active_paper=i;
     paper[0]=new Paper(scrollview->viewport(),AppControl->Prefs->Rect("PaperWidth","PaperHeight"),AppControl);
     scrollview->addChild(paper[0]);
     paper[0]->show();
   }
}



void MathWindow::Calculate()
{
int i;
for(i=0;i<32;i++) {
if (paper[i]) {
	paper[i]->UpDate();
}
}
}




void MathWindow::Save(QString file)
{	
	
	datastore=new DataStorage();
	
	DataStorage *ds=AppControl->Prefs->Build();
	datastore->AddDataStorage("prefs",ds,false);
	delete ds;
	
	ds=paper[0]->BuildStorage();
	datastore->AddDataStorage("page#0",ds,false);
	delete ds;
		
	char *rawdata;
	int size;
	datastore->Record(&rawdata,&size,true);
	
	QFile f(file);
	f.open( IO_WriteOnly );
	f.writeBlock(rawdata,(unsigned int)size);
	
	free(rawdata);
	f.close();
	
	delete datastore;
	CurrentDocumentName=file;
	setCaption(ApplicationCaption+QString(" / ")+QFileInfo(f).fileName());
	AppControl->DocumentChanged=false;	
}



void MathWindow::Open(QString file)
{
	DataStorage *ds;
	char *rawdata=0;
  bool loadprefs=true;
	Bug("** Opening File **");
	QFile f(file);
	if (f.exists()==false)  {
                       loadprefs=false;
                       if (file!=config_path+QString("/MathPlanner-autosave.mpl2")) ErrorReport("Open File","Requested file not found...");
                 }
	else {
	f.open( IO_ReadOnly );
	int siz=f.size();
	
	rawdata=(char *)malloc(siz+16);	
	if (!rawdata) ErrorReport("Open","Memory error");
	f.readBlock(rawdata,siz);
	
	
	datastore=new DataStorage();
	
	datastore->Build(rawdata,siz);	
	
	free(rawdata);
	
	paper[0]->New();
	
  if (datastore->Contains("prefs") && loadprefs && AppControl->Prefs->Int("AlwaysDefault")==0) {
	ds=new DataStorage();
	datastore->ReadDataStorage("prefs",ds);
	Bug("Start recording prefs");
	AppControl->Prefs->Record(ds);
	Bug("Prefs recorded");
	delete ds;
  }
	
	ds=new DataStorage();
	datastore->ReadDataStorage("page#0",ds);
	paper[0]->Record(ds);	
	delete ds;
	delete datastore;
	
  paper[0]->UpDate();

	AppControl->DocumentChanged=false;
	
	CurrentDocumentName=file;
	
	setCaption(ApplicationCaption+QString(" / ")+QFileInfo(f).fileName());
	
	Bug("*****  Loading  Completed  *****");
	}
}
















