/*
 * MathPlanner 3.1 - Mathematical design tool.
 * Copyright(C) 2002  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */

#ifndef _MATHPLANNER_PAPER_H
#define _MATHPLANNER_PAPER_H


#include <qpoint.h>
#include <qrect.h>
#include <qdragobject.h>
#include <qevent.h>
#include <qimage.h>
#include <qpainter.h>
#include <qwidget.h>


class MathFrame;
class ApplicationControl;
class NMemory;
class config_file_reader;
class DataStorage;






class Paper : public QWidget
{

  private:
  bool    exclude_i;
  int     Focused_function;
  int     *Order;
  int     Function_count;
  double  WorldScaleX,WorldScaleY;
  QPoint  *FunctionPosition;
  QPoint  Begin_move_align;
  bool    Begin_move,Begin_resize;
  QRect 	PaperLimits,Printable;
  int     Grid;
  QColor 	GridColor;

  class   MathFrame **function;
  class   NMemory *function_mem;
  class   NMemory *Order_mem;
  class   NMemory *Position_mem;

  public:

  class config_file_reader *Prefs;
  class ApplicationControl *AppControl;

  Paper(QWidget *,QRect,class ApplicationControl *);

  void         CalculateAll();

  DataStorage *BuildStorage();

  void      dragEnterEvent(QDragEnterEvent *);
  void      dropEvent(QDropEvent *);
  void      DragObject();
  void      DropObject();
  QImage    CreateImage(class MathFrame *);
  void      StartDragging();
  void      Record(DataStorage *);
  void      New();
  void      UpDate();
  void      SetNewFunctionAfter(class MathFrame *);
  void      DrawFunction(class MathFrame *);
  QRect     Scale(int zoom,QRect);
  QPoint    ScreenToDocument(QPoint);
  QPoint    FixedFunctionPosition(MathFrame *,bool pm=false);
  QRect     ScreenToDocument(QRect);
  void      FocusNone();
  void      UnFocusFunction(int no);
  void      UnFocusCurrentFunction();
  void      FocusFunction(int no);
  void      DeleteCurrentFunction();
  void      CommandToString(int,int);
  void      Draw(QPainter *,QRect,bool);
  class     MathFrame *GetFocus();
  class     m_function_object *GetFunctionFocus();
  void      AddGraph();

  void      keyPressEvent(class QKeyEvent *);

  private:

  void  Snap(QPoint p,int *x,int *y);
  void  DeleteEmptyFunctions();
  void  SetFunctionSpace(int);
  void  AddFunctionSpace(int);
  int   FindFunctionIndex(class MathFrame *);
  void  AddFunction(QPoint p,class MathFrame *,bool flag=false);
  void  DeleteFunction(int i);
  bool  CheckPoint(QPoint);
  void  mousePressEvent(class QMouseEvent *);
  void  mouseReleaseEvent(class QMouseEvent *);
  void  mouseMoveEvent(class QMouseEvent *);
  void  paintEvent(class QPaintEvent *);
  void  DrawGrid(QPainter *);
  int   OrderFunctions();
};

#endif


