
#include "Prefs.h"

#include <qsize.h>
#include <qfont.h>
#include <qfontdialog.h>
#include <qcolordialog.h>


#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qmessagebox.h>


#include "ConfigReader.h"
#include "Setup.moc"
#include "Prefs.moc"


setup::~setup()
{
delete Dialog;
}


setup::setup(QWidget *parent,config_file_reader *Pr)
:QObject(parent)
{

Prefs=Pr;

Dialog=new Preferences(parent,NULL,0,0);
Dialog->hide();

QSize size=QSize(20,14);

bm1.resize(size);
bm2.resize(size);
bm3.resize(size);
bm4.resize(size);
bm5.resize(size);

fontb=QFont(Prefs->String("BaseFont"),12,Prefs->Int("BaseFontWeight"),(bool)Prefs->Int("BaseFontItalic"));
fonto=QFont(Prefs->String("OperatorFont"),12,Prefs->Int("OperatorFontWeight"),(bool)Prefs->Int("OperatorFontItalic"));
fonti=QFont(Prefs->String("IndexFont"),12,Prefs->Int("IndexFontWeight"),(bool)Prefs->Int("IndexFontItalic"));
fontf=QFont(Prefs->String("FunctionFont"),12,Prefs->Int("FunctionFontWeight"),(bool)Prefs->Int("FunctionFontItalic"));

connect(Dialog->Color_B,SIGNAL(clicked()),this,SLOT(ColorB()));
connect(Dialog->Color_I,SIGNAL(clicked()),this,SLOT(ColorI()));
connect(Dialog->Color_F,SIGNAL(clicked()),this,SLOT(ColorF()));
connect(Dialog->Color_O,SIGNAL(clicked()),this,SLOT(ColorO()));
connect(Dialog->Color_S,SIGNAL(clicked()),this,SLOT(ColorS()));

connect(Dialog->Set_B,SIGNAL(clicked()),this,SLOT(FontB()));
connect(Dialog->Set_I,SIGNAL(clicked()),this,SLOT(FontI()));
connect(Dialog->Set_F,SIGNAL(clicked()),this,SLOT(FontF()));
connect(Dialog->Set_O,SIGNAL(clicked()),this,SLOT(FontO()));


connect(Dialog->Ok,SIGNAL(clicked()),this,SLOT(Ok()));

}


QString setup::FontInfo(QFont font)
{
QString fi=font.family();
if (font.italic()) fi.append(", italic, ");
else fi.append(", ");
QString no;
no.setNum(font.weight());
fi.append(no);
return(fi);
}


void setup::Show()
{
Init();
Dialog->show();
}



void setup::Init()
{
Dialog->Family_B->setText(FontInfo(fontb));
Dialog->Family_I->setText(FontInfo(fonti));
Dialog->Family_F->setText(FontInfo(fontf));
Dialog->Family_O->setText(FontInfo(fonto));

co1=Prefs->Color("BaseFontColor");
co2=Prefs->Color("IndexFontColor");
co3=Prefs->Color("FunctionFontColor");
co4=Prefs->Color("OperatorFontColor");
co5=Prefs->Color("SymbolFontColor");

bm1.fill(co1);
bm2.fill(co2);
bm3.fill(co3);
bm4.fill(co4);
bm5.fill(co5);

Dialog->Color_B->setPixmap(bm1);
Dialog->Color_I->setPixmap(bm2);
Dialog->Color_F->setPixmap(bm3);
Dialog->Color_O->setPixmap(bm4);
Dialog->Color_S->setPixmap(bm5);

Dialog->Size_N->setValue(Prefs->Int("FontSizeNormal"));
Dialog->Size_M->setValue(Prefs->Int("FontSizeSmall"));
Dialog->Size_S->setValue(Prefs->Int("FontSizeExpSmall"));
Dialog->CenterLine->setValue(Prefs->Int("BaseFontCenterLine"));

}



void setup::Ok()
{
Prefs->SetInt("FontSizeNormal",Dialog->Size_N->value());
Prefs->SetInt("FontSizeSmall",Dialog->Size_M->value());
Prefs->SetInt("FontSizeIndex",Dialog->Size_M->value());
Prefs->SetInt("FontSizeExpSmall",Dialog->Size_S->value());
Prefs->SetInt("FontSizeIndexSmall",Dialog->Size_S->value());
Prefs->SetInt("BaseFontCenterLine",Dialog->CenterLine->value());

Prefs->CreateFile();
Dialog->hide();

QMessageBox::information( 0, "Preferences",QObject::tr("You must now restart application"));

}


void setup::ColorB()
{
QColor c=QColorDialog::getColor(co1);
if (c.isValid()) {
bm1.fill(c);
Dialog->Color_B->setPixmap(bm1);
Prefs->SetColor("BaseFontColor",c);
}
}

void setup::ColorI()
{
QColor c=QColorDialog::getColor(co2);
if (c.isValid()) {
bm2.fill(c);
Dialog->Color_I->setPixmap(bm2);
Prefs->SetColor("IndexFontColor",c);
}
}

void setup::ColorF()
{
QColor c=QColorDialog::getColor(co3);
if (c.isValid()) {
bm3.fill(c);
Dialog->Color_F->setPixmap(bm3);
Prefs->SetColor("FunctionFontColor",c);
}
}

void setup::ColorO()
{
QColor c=QColorDialog::getColor(co4);
if (c.isValid()) {
bm4.fill(c);
Dialog->Color_O->setPixmap(bm4);
Prefs->SetColor("OperatorFontColor",c);
}
}

void setup::ColorS()
{
QColor c=QColorDialog::getColor(co5);
if (c.isValid()) {
bm5.fill(c);
Dialog->Color_S->setPixmap(bm5);
Prefs->SetColor("SymbolFontColor",c);
}
}






void setup::FontB()
{
bool ok;
QFont font=QFontDialog::getFont(&ok, fontb);
if (ok) {
   fontb=font;
   Dialog->Family_B->setText(FontInfo(font));
   QString fam=font.family();
   int i=fam.find("[");
   if (i!=-1) fam.truncate(i-1);
   Prefs->SetString("BaseFont",fam);
   Prefs->SetInt("BaseFontItalic",font.italic());
   Prefs->SetInt("BaseFontWeight",font.weight());
}
}

void setup::FontI()
{
bool ok;
QFont font=QFontDialog::getFont(&ok, fonti);
if (ok) {
   fonti=font;
   Dialog->Family_I->setText(FontInfo(font));
   QString fam=font.family();
   int i=fam.find("[");
   if (i!=-1) fam.truncate(i-1);
   Prefs->SetString("IndexFont",fam);
   Prefs->SetInt("IndexFontItalic",font.italic());
   Prefs->SetInt("IndexFontWeight",font.weight());
}
}

void setup::FontF()
{
bool ok;
QFont font=QFontDialog::getFont(&ok, fontf);
if (ok) {
   fontf=font;
   Dialog->Family_F->setText(FontInfo(font));
   QString fam=font.family();
   int i=fam.find("[");
   if (i!=-1) fam.truncate(i-1);
   Prefs->SetString("FunctionFont",fam);
   Prefs->SetInt("FunctionFontItalic",font.italic());
   Prefs->SetInt("FunctionFontWeight",font.weight());
}
}


void setup::FontO()
{
bool ok;
QFont font=QFontDialog::getFont(&ok, fonto);
if (ok) {
   fonto=font;
   Dialog->Family_O->setText(FontInfo(font));
   QString fam=font.family();
   int i=fam.find("[");
   if (i!=-1) fam.truncate(i-1);
   Prefs->SetString("OperatorFont",fam);
   Prefs->SetInt("OperatorFontItalic",font.italic());
   Prefs->SetInt("OperatorFontWeight",font.weight());
}
}











